/****************************************************************************

 Title	:   C include file for the HD44780U LCD library (lcd.c)

 Compiler: >->->->->->->->  GCCAVR  <-<-<-<-<-<-<-<-<-<-<-<-<


 CHANGED BY R. Muench Oct-1-2002 

 # reduced to 4bit mode only
 # independent Control lines RS RW E LCDPOWER implemented
 # user defined character implemented
 # timing corrected to meet specifications
 # write data/command handler reduced to minimum
 # easy use for lcd types - see lcd.h


 

 Author:    Peter Fleury <pfleury@gmx.ch>  http://jump.to/fleury
 Date:	    23-Sep-2000
 Software:  AVR-GCC with AVR-AS
 Target:    any AVR device, memory mapped mode only for AT90S4414/8515/Mega

 DESCRIPTION
       Basic routines for interfacing a HD44780U-based text lcd display

       Based on Volker Oth's lcd library (http://members.xoom.com/volkeroth),
       changed lcd_init(), added additional constants for lcd_command(),
       added 4-bit I/O mode, improved and optimized code.
       

 USAGE
       See the C include lcd.h file for a description of each function
       

*****************************************************************************/
#include "include_all.h"			// lcd config 


/* 
** constants/macros 
*/
#define DDR(x) ((x)-1)    /* address of data direction register of port x */
#define PIN(x) ((x)-2)    /* address of input register of port x */

#define lcd_e_delay()   asm volatile("rjmp _PC_+0")  /* delay 500ns with 4Mhz */

#define lcd_on()        sbi(LCD_PWR_PORT, LCD_PWR_PIN)
#define lcd_off()       cbi(LCD_PWR_PORT, LCD_PWR_PIN)
#define lcd_pwr_init()  sbi(LCD_PWR_PORT-1, LCD_PWR_PIN)

#define lcd_rw_high()   sbi(LCD_RW_PORT, LCD_RW_PIN)
#define lcd_rw_low()    cbi(LCD_RW_PORT, LCD_RW_PIN)
#define lcd_rw_init()   sbi(LCD_RW_PORT-1, LCD_RW_PIN)

#define lcd_rs_high()   sbi(LCD_RS_PORT, LCD_RS_PIN)
#define lcd_rs_low()    cbi(LCD_RS_PORT, LCD_RS_PIN)
#define lcd_rs_init()   sbi(LCD_RS_PORT-1, LCD_RS_PIN)

#define lcd_e_high()    sbi(LCD_E_PORT, LCD_E_PIN)
#define lcd_e_low()     cbi(LCD_E_PORT, LCD_E_PIN)
#define lcd_e_init()    sbi(LCD_E_PORT-1, LCD_E_PIN)
#define lcd_e_toggle()  toggle_e()


#if LCD_LINES==1
#define LCD_FUNCTION_DEFAULT    LCD_FUNCTION_4BIT_1LINE 
#else
#define LCD_FUNCTION_DEFAULT    LCD_FUNCTION_4BIT_2LINES 
#endif

/* 
** function prototypes 
*/
static void delay(u16 us);
static void toggle_e(void);

/*
** local functions
*/

/*-----------------------------------------------------------------------------*/
static void delay(u16 us) 
/* delay for a minimum of <us> microseconds    */
/* with a 4Mhz crystal, the resolution is 1 us */
{
    while ( us ) us--;
} 


/*-----------------------------------------------------------------------------*/
static void toggle_e(void)
/* toggle Enable Pin */
{
    lcd_e_high();
    delay(lcddelay_1us);  /* wait 1us */
    lcd_e_low();
}


/*-----------------------------------------------------------------------------*/
static void inline lcd_write(u08 data,u08 rs) 
{
    register u08 temp;
    
	temp = 0x0F | inp(DDR(LCD_DATA_PORT));
	outp(temp, DDR(LCD_DATA_PORT));


	temp = inp(LCD_DATA_PORT) & 0xF0;		// load controll data 
    outp(temp | (data >> 4),LCD_DATA_PORT);	// send controll data once

	lcd_rw_low();

	if(rs)
		lcd_rs_high();	// data
	else
		lcd_rs_low();	// command
			
    lcd_e_toggle();

    temp = inp(LCD_DATA_PORT) & 0xF0;		// load controll data 
    outp(temp | (data & 0x0F),LCD_DATA_PORT);	// send controll data once

    lcd_e_toggle();

}


/*-----------------------------------------------------------------------------*/
static u08 lcd_read(u08 rs) 
{
    register u08 dataH, dataL,temp;

    if (rs) sbi(LCD_RS_PORT, LCD_RS_PIN);    /* RS=1: read data      */
    else    cbi(LCD_RS_PORT, LCD_RS_PIN);    /* RS=0: read busy flag */
    sbi(LCD_RW_PORT, LCD_RW_PIN);            /* RW=1  read mode      */

    /* configure data pins as input */
    temp = 0xF0 & inp(DDR(LCD_DATA_PORT));
    outp(temp, DDR(LCD_DATA_PORT));
    
    lcd_e_high();
    delay(lcddelay_1us);  /* wait 1us */
    dataH = inp(PIN(LCD_DATA_PORT));         /* read high nibble first */

    lcd_e_low();
    delay(lcddelay_1us);  /* wait 1us */

    lcd_e_high();
    delay(lcddelay_1us);  /* wait 1us */

    dataL = inp(PIN(LCD_DATA_PORT));         /* read low nibble        */
    lcd_e_low();
    delay(lcddelay_1us);  /* wait 1us */
    
    return ( (dataH<<4) | (dataL&0x0F) );
}



/*-----------------------------------------------------------------------------*/
static inline unsigned char lcd_waitbusy(void)
/* loops while lcd is busy, reads address counter */
{
    register unsigned char c;
    
    while ( (c=lcd_read(0)) & (1<<LCD_BUSY)) {}

    return (c);  // return address counter
}


/*-----------------------------------------------------------------------------*/
static inline void lcd_newline(u08 pos)
/* goto start of next line */
{
    register u08 addressCounter;

#if LCD_LINES==1
    addressCounter = 0;
#endif
#if LCD_LINES==2
    if ( pos < (LCD_START_LINE2) )
        addressCounter = LCD_START_LINE2;
    else
        addressCounter = LCD_START_LINE1;
#endif
#if LCD_LINES==4
    if ( pos < LCD_START_LINE3 )
        addressCounter = LCD_START_LINE2;
    else if ( (pos >= LCD_START_LINE2) && (pos < LCD_START_LINE4) )
        addressCounter = LCD_START_LINE3;
    else if ( (pos >= LCD_START_LINE3) && (pos < LCD_START_LINE2) )
        addressCounter = LCD_START_LINE4;
    else 
        addressCounter = LCD_START_LINE1;
#endif

    lcd_command((1<<LCD_DDRAM)+addressCounter);

}/* lcd_newline */


/*
** PUBLIC FUNCTIONS 
*/

/*-----------------------------------------------------------------------------*/
void lcd_command(u08 cmd)
/* send commando <cmd> to LCD */
{
    lcd_waitbusy();
    lcd_write(cmd,0);
}


/*-----------------------------------------------------------------------------*/
void lcd_gotoxy(u08 x, u08 y)
/* goto position (x,y) */
{
#if LCD_LINES==1
    lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
#endif
#if LCD_LINES==2
    if ( y==0 ) 
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE2+x);
#endif
#if LCD_LINES==4
    if ( y==0 )
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+x);
    else if ( y==1)
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE2+x);
    else if ( y==2)
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE3+x);
    else /* y==3 */
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE4+x);
#endif

}/* lcd_gotoxy */


/*-----------------------------------------------------------------------------*/
void lcd_clrscr(void)
/* clear lcd and set cursor to home position */
{
    lcd_command(1<<LCD_CLR);
}


/*-----------------------------------------------------------------------------*/
void lcd_home(void)
/* set cursor to home position */
{
    lcd_command(1<<LCD_HOME);
}


/*-----------------------------------------------------------------------------*/
/*
	control codes 	\n : newline   	set cursor in first pos one line deeper 
					\f : formfeed  	clearing the display

	codes over 0x80 are limited to 0x00..0x7F
	so the user defined codes are reachable with 0x80..0x87 - see lcdfont.h

*/

void lcd_putc(char c)
/* print character at current cursor position */
{
    register unsigned char pos;
    
    pos = lcd_waitbusy();   // read busy-flag and address counter

    if (c=='\f'){
    		lcd_command(1<<LCD_CLR);
    		pos = lcd_waitbusy();   // read busy-flag and address counter
    		return;
    	}

#if LCD_241==1			/* special lcd type 161 */

    if (c=='\n')
    	lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1);
 	
    else
    {
    	if(pos==LCD_1END)	
    		lcd_command((1<<LCD_DDRAM)+LCD_START_LINE2);

        pos = lcd_waitbusy();   // read busy-flag and address counter

		lcd_write(0x7F&c, 1);
    }


#else
    if (c=='\n')
        lcd_newline(pos);
    else
		lcd_write(0x7F&c, 1);
#endif
}


/*-----------------------------------------------------------------------------*/
void lcd_puts_r(char *s)
/* print string on lcd (no auto linefeed) */
{
    register char c;

    while ( (c = *s++) ) {
        lcd_putc(c);
    }

}


/*-----------------------------------------------------------------------------*/
/** USE lcd_puts_P("text") **/

void lcd_puts_f(const char *progmem_s)
/* print string from program memory on lcd (no auto linefeed) */
{
    register char c;

    while ( (c = PRG_RDB(progmem_s++)) ) {
        lcd_putc(c);
    }
}

/*-----------------------------------------------------------------------------*/
void lcd_init(u08 dispAttr)
/* initialize display and select type of cursor */
/* dispAttr: LCD_DISP_OFF, LCD_DISP_ON, LCD_DISP_ON_CURSOR, LCD_DISP_CURSOR_BLINK */
{
    register u08 temp, i;
    
    /*------ Initialize lcd to 4 bit i/o mode -------*/

    lcd_rs_init();
    lcd_rw_init();
    lcd_e_init();
    	
#if LCD_POWERCONTROL		/* using off extra powerpin for lcd supply */
    lcd_pwr_init();
    lcd_on();
#endif

    temp = 0x0F | inp(DDR(LCD_DATA_PORT));
    outp(temp, DDR(LCD_DATA_PORT));

	for(i=0;i<30;i++)			/* 30 times wait time for more than 40ms */
	    delay(lcddelay_2ms);  	/* wait 2ms after power-on */
    

//---------  initial write to lcd is 8bit -----------------

	for(i=0;i<2;i++){	/* send init twice */
	
    	temp = inp(LCD_DATA_PORT) & 0xF0;		// load controll data 
    	outp(temp | (LCD_FUNCTION_SET >> 4),LCD_DATA_PORT);	// send controll data once

    	lcd_e_toggle();

    	temp = inp(LCD_DATA_PORT) & 0xF0;		// load controll data 
    	outp(temp | (LCD_FUNCTION_SET & 0x0F),LCD_DATA_PORT);	// send controll data once

    	lcd_e_toggle();
    	delay(lcddelay_2ms);  			// wait 2ms 
    };
    
//---------  from now the lcd only accepts 4 bit I/O, we can use lcd_command() ----

    lcd_command(LCD_DISP_OFF);              /* display off                  */
    lcd_clrscr();                           /* display clear                */ 
    lcd_command(LCD_MODE_DEFAULT);          /* set entry mode               */
    lcd_command(dispAttr);                  /* display/cursor control       */

#if LCD_USERFONT == ON
	lcd_setuserfont_f(USERFONT);			/* set user defined character set */
#endif	

}/* lcd_init */

/*-----------------------------------------------------------------------------*/
/* set font from sram (runtime changeable) */
//void lcd_setuserfont_r(volatile u08 *font)
void lcd_setuserfont_r(u08 *font)
{
    register char c,i;

	lcd_command((1<<LCD_CGRAM));	// open CGRAM

	for(i=0;i<64;i++){				// transmit 64 bytes

		c = *font++;
    	lcd_waitbusy();   			// read busy-flag
		lcd_write(c, 1);
    }

	lcd_command((1<<LCD_DDRAM));	// close CGRAM

}/*   lcd_setuserfont_r */

/*-----------------------------------------------------------------------------*/
/* set font from program memory (fix font) */
void lcd_setuserfont_f(const char *font)
{
    register char c,i;

	lcd_command((1<<LCD_CGRAM));	// open CGRAM

	for(i=0;i<64;i++){				// transmit 64 bytes

    	c = PRG_RDB(font++);
    	lcd_waitbusy();   			// read busy-flag
		lcd_write(c, 1);
    }

	lcd_command((1<<LCD_DDRAM));	// close CGRAM

}/*   lcd_setuserfont_p */

/*-----------------------------------------------------------------------------*/
/* lcd.c */