/*************************************************************************
 
 Title	:   C include file for the HD44780U LCD library (lcd.c)

 Compiler: >->->->->->->->  GCCAVR  <-<-<-<-<-<-<-<-<-<-<-<-<


 CHANGED BY R. Muench @ Oct-1-2002 <muench@behnk.de>

 # reduced to 4bit mode only
 # independent Control lines RS RW E LCDPOWER implemented (port bits which ever you want)
 # user defined character implemented	(8 characters runtime or fix defined) see lcdfont.h & LCD_USERFONT
 # timing corrected to meet specifications (based on Displaytech TM)
 # write data/command handler reduced to minimum
 # easy use for lcd types - see LCD_TYPE	(specify which lcd type you want to use)
 


 invented by:   Peter Fleury <pfleury@gmx.ch>  http://jump.to/fleury
 				Date:	    23-Sep-2000
 				Software:  AVR-GCC with AVR-AS
 				Target:    any AVR device

 DESCRIPTION
       Basic routines for interfacing a HD44780U-based text lcd display

       Based on Volker Oth's lcd library (http://members.xoom.com/volkeroth),
       changed lcd_init(), added additional constants for lcd_command(), 
       added 4-bit I/O mode, improved and optimized code.
       
 

***************************************************************************/
#ifndef LCD_H
#define LCD_H


/**********************************************************************************
 time setting for LCD	CHANGE XTAL FOR DESIRED SYSTEM SPEED
 ---------------------------------------------------------------------------------*/
 
//#define XTAL 6553600				/* CPU-Frequence in Hz */

#define lcddelay_2ms  XTAL*2000E-6/4+1		/* initilize delay */
#define lcddelay_1us  XTAL*1E-6/4+1			/* datarate delay  */

/**********************************************************************************
 port setting for LCD	CHANGE FOR YOUR SYSTEM
 
 # supports independent portbit alignment for controll lines 
 # only the lower 4 data lines have to be aligned at one port 
   and tied to higher nibble on lcd
 # supports LCD-Power switch function via on pin (direct pin supply)
   so battery applications are possible
 ---------------------------------------------------------------------------------*/

#define LCD_DATA_PORT    PORTB     /* port for 4bit data[0..3] -> LCD[4..7] */
#define LCD_RS_PORT      PORTC     /* port for RS line */
#define LCD_RS_PIN       2
#define LCD_RW_PORT      PORTD     /* port for RW line */
#define LCD_RW_PIN       6
#define LCD_E_PORT       PORTB     /* port for Enable line */
#define LCD_E_PIN        4

#define LCD_POWERCONTROL ON	   /* */

#if LCD_POWERCONTROL == ON
#define LCD_PWR_PORT     PORTD     /* port for Power */
#define LCD_PWR_PIN      7
#endif

/***********************************************************************************
 display type setting	CHANGE LCDTYPE FOR YOUR SYSTEM

 This makes it easier to handle all the different types of lcd's


 SYNTAX:  161 : [CHARACTERCOUNT PER LINE][LINECOUNT] 

 example  161 means 16character 1 line
 example  202 means 20character 2 lines

 
 supported types
 
  81	 8 Characters 1 Line
  82	 8 Characters 2 Lines
 122	12 Characters 2 Lines
 161	16 Characters 1 Line
 162	16 Characters 2 Lines
 164	16 Characters 4 Lines
 201	20 Characters 1 Line
 202	20 Characters 2 Lines
 204	20 Characters 4 Lines
 241	24 Characters 1 Line
 321	32 Characters 1 Line
 401	40 Characters 1 Line
 402	40 Characters 2 Lines	*/


//#define LCDTYPE			161			/* LCD-Modul type **** CHANGE HERE **** */

#define	LCD_DL 			OFF			/* OFF = data bus length 4bit */
#define	LCD_F  			OFF			/* OFF = font 5x7 */


/*-----------------------------------------------------------------------*/
// Predefined LCD types
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 8 characters in 1 line

    [00|01|02|03|04|05|06|07]	DD address aligment
*/

#if LCDTYPE == 81	/* 8 characters 1 line */

#define LCD_N 				OFF		/* 1 line */	
#define LCD_241     		OFF   	/* two for one */
#define LCD_LINES       	1   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 8 characters in 2 lines

    [00|01|02|03|04|05|06|07]	DD address alignment line 1
    [40|41|42|43|44|45|46|47]	DD address alignment line 2
*/

#if LCDTYPE == 82	/* 8 characters 2 lines */

#define LCD_N  		    	ON		/* 2 lines */	
#define LCD_LINES           2   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */
#define LCD_START_LINE2 	0x40   	/* DDRAM address of first char of line 2 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 12 characters in 2 lines

    [00|01|02|03|04|05|06|07|08|09|0A|0B]	DD address alignment line 1
    [40|41|42|43|44|45|46|47|48|49|4A|4B]	DD address alignment line 2

*/
#if LCDTYPE == 122	/* 12 characters 2 lines */

#define LCD_N  		    	ON		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           2  		/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */
#define LCD_START_LINE2  	0x40   	/* DDRAM address of first char of line 2 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 16 characters in 1 lines

    [00|01|02|03|04|05|06|07|40|41|42|43|44|45|46|47]	DD address alignment
                             ^^
                             !!
                             
    this is real a 8 character 2 line display as type 82 before, but in one line!
*/

#if LCDTYPE == 161	/* 16 characters 1 line */

#define LCD_N  		    	ON		/* 2 lines */	
#define LCD_241             ON  	/* two for one */
#define LCD_1END	    	8		/* invisible line 1 end and start off line 2 as digit 9*/	
#define LCD_LINES           2   	/* visible lines */  /* carfully! 2 ist right here! */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */
#define LCD_START_LINE2 	0x40   	/* DDRAM address of first char of line 2 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 16 characters in 2 lines

    [00|01|02|03|04|05|06|07|08|09|0A|0B|0C|0D|0E|0F]	DD address alignment line 1
    [40|41|42|43|44|45|46|47|48|49|4A|4B|4C|4D|4E|4F]	DD address alignment line 2

*/
#if LCDTYPE == 162	/* 16 characters 2 lines */

#define LCD_N  		    	ON		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           2   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */
#define LCD_START_LINE2  	0x40   	/* DDRAM address of first char of line 2 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 16 characters in 4 lines

    [00|01|02|03|04|05|06|07|08|09|0A|0B|0C|0D|0E|0F]	DD address alignment line 1
    [10|11|12|13|14|15|16|17|18|19|1A|1B|1C|1D|1E|1F]	DD address alignment line 2
    [40|41|42|43|44|45|46|47|48|49|4A|4B|4C|4D|4E|4F]	DD address alignment line 3
    [50|51|52|53|54|55|56|57|58|59|5A|5B|5C|5D|5E|5F]	DD address alignment line 4

*/
#if LCDTYPE == 164	/* 16 characters 4 lines */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           4   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */
#define LCD_START_LINE2  	0x40   	/* DDRAM address of first char of line 2 */
#define LCD_START_LINE3  	0x10   	/* DDRAM address of first char of line 3 */
#define LCD_START_LINE4  	0x50   	/* DDRAM address of first char of line 4 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 20 characters in 1 line

    [00|01|02|03|04|05|06|07|08|09|0A|0B|0C|0D|0E|0F|10|11|12|13]	DD address alignment

*/
#if LCDTYPE == 201	/* 20 characters 1 line */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           1   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 20 characters in 2 lines

    [00|01|02|03|04|05|06|07|..|0F|10|11|12|13]	DD address alignment line 1
    [40|41|42|43|44|45|46|47|..|4F|50|51|52|53]	DD address alignment line 2

*/
#if LCDTYPE == 202	/* 20 characters 2 lines */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           2  		/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */
#define LCD_START_LINE2  	0x40   	/* DDRAM address of first char of line 2 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 20 characters in 4 lines

    [00|01|02|03|04|05|06|07|..|0F|10|11|12|13]	DD address alignment line 1
    [40|41|42|43|44|45|46|47|..|4F|50|51|52|53]	DD address alignment line 2
    [14|15|16|17|18|19|1A|1B|..|23|24|25|26|27] DD address alignment line 3
    [54|55|56|57|58|59|5A|5B|..|63|64|65|66|67]	DD address alignment line 4

*/
#if LCDTYPE == 204	/* 20 characters 4 lines */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           4  		/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */
#define LCD_START_LINE2  	0x40   	/* DDRAM address of first char of line 2 */
#define LCD_START_LINE1  	0x14   	/* DDRAM address of first char of line 3 */
#define LCD_START_LINE2  	0x54   	/* DDRAM address of first char of line 4 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 24 characters in 1 line

    [00|01|02|03|04|05|06|07|..|11|12|13|14|15|16|17]	DD address alignment

*/
#if LCDTYPE == 241	/* 24 characters 1 line */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           1   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 24 characters in 2 lines

    [00|01|02|03|04|05|06|07|..|11|12|13|14|15|16|17]	DD address alignment line 1
    [40|41|42|43|44|45|46|47|..|51|52|53|54|55|56|57]	DD address alignment line 2

*/
#if LCDTYPE == 242	/* 24 characters 2 lines */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           2   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 32 characters in 1 line

    [00|01|02|03|04|05|06|07|..|19|1A|1B|1C|1D|1E|1F]	DD address alignment

*/
#if LCDTYPE == 321	/* 32 characters 1 line */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           1   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 40 characters in 1 line

    [00|01|02|03|04|05|06|07|..|22|23|24|25|26|27]	DD address alignment

*/
#if LCDTYPE == 401	/* 40 characters 1 line */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           2   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */

#endif
/*-----------------------------------------------------------------------*/
/*
	alphanumeric LCD 40 characters in 2 lines

    [00|01|02|03|04|05|06|07|..|23|24|25|26|27]	DD address alignment line 1
    [40|41|42|43|44|45|46|47|..|63|64|65|66|67]	DD address alignment line 2

*/
#if LCDTYPE == 402	/* 40 characters 2 lines */

#define LCD_N  		    	0		/* 1 line */	
#define LCD_241             OFF 	/* two for one */
#define LCD_LINES           2   	/* visible lines */
#define LCD_LINE_LENGTH  	0x40   	/* internal line length */
#define LCD_START_LINE1  	0x00   	/* DDRAM address of first char of line 1 */
#define LCD_START_LINE2  	0x40   	/* DDRAM address of first char of line 2 */

#endif
/*-----------------------------------------------------------------------*/


#define	LCD_FUNCTION_SET ( 0x20 | (LCD_DL<<4) | (LCD_F<<2) | (LCD_N<<3) )



// ************************************************************************
/* you shouldn't need to change anything below this line */

typedef unsigned char  u08;
typedef unsigned short u16;


/* instruction register bit positions */
#define LCD_CLR             0      /* DB0: clear display */
#define LCD_HOME            1      /* DB1: return to home position */
#define LCD_ENTRY_MODE      2      /* DB2: set entry mode */
#define LCD_ENTRY_INC       1      /*   DB1: 1=increment, 0=decrement  */
#define LCD_ENTRY_SHIFT     0      /*   DB2: 1=display shift on        */
#define LCD_ON              3      /* DB3: turn lcd/cursor on */
#define LCD_ON_DISPLAY      2      /*   DB2: turn display on */
#define LCD_ON_CURSOR       1      /*   DB1: turn cursor on */
#define LCD_ON_BLINK        0      /*     DB0: blinking cursor ? */
#define LCD_MOVE            4      /* DB4: move cursor/display */
#define LCD_MOVE_DISP       3      /*   DB3: move display (0-> cursor) ? */
#define LCD_MOVE_RIGHT      2      /*   DB2: move right (0-> left) ? */
#define LCD_FUNCTION        5      /* DB5: function set */
#define LCD_FUNCTION_8BIT   4      /*   DB4: set 8BIT mode (0->4BIT mode) */
#define LCD_FUNCTION_2LINES 3      /*   DB3: two lines (0->one line) */
#define LCD_FUNCTION_10DOTS 2      /*   DB2: 5x10 font (0->5x7 font) */
#define LCD_CGRAM           6      /* DB6: set CG RAM address */
#define LCD_DDRAM           7      /* DB7: set DD RAM address */
#define LCD_BUSY            7      /* DB7: LCD is busy */

/* set entry mode: display shift on/off, dec/inc cursor move direction */
#define LCD_ENTRY_DEC            0x04   /* display shift off, dec cursor move dir */
#define LCD_ENTRY_DEC_SHIFT      0x05   /* display shift on,  dec cursor move dir */
#define LCD_ENTRY_INC_           0x06   /* display shift off, inc cursor move dir */
#define LCD_ENTRY_INC_SHIFT      0x07   /* display shift on,  inc cursor move dir */

/* display on/off, cursor on/off, blinking char at cursor position */
#define LCD_DISP_OFF             0x08   /* display off                            */
#define LCD_DISP_ON              0x0C   /* display on, cursor off                 */
#define LCD_DISP_ON_BLINK        0x0D   /* display on, cursor off, blink char     */
#define LCD_DISP_ON_CURSOR       0x0E   /* display on, cursor on                  */
#define LCD_DISP_ON_CURSOR_BLINK 0x0F   /* display on, cursor on, blink char      */

/* move cursor/shift display */
#define LCD_MOVE_CURSOR_LEFT     0x10   /* move cursor left  (decrement)          */
#define LCD_MOVE_CURSOR_RIGHT    0x14   /* move cursor right (increment)          */
#define LCD_MOVE_DISP_LEFT       0x18   /* shift display left                     */
#define LCD_MOVE_DISP_RIGHT      0x1C   /* shift display right                    */


#define LCD_MODE_DEFAULT     ((1<<LCD_ENTRY_MODE) | (1<<LCD_ENTRY_INC) )


/* 
** function prototypes 
*/
extern void lcd_command(u08 cmd);
extern void lcd_gotoxy(u08 x, u08 y);
extern void lcd_clrscr(void);
extern void lcd_home(void);
extern void lcd_putc(char c);
extern void lcd_puts_r(char *s);
extern void lcd_puts_f(const char *progmem_s);
extern void lcd_init(u08 dispAttr);
//extern void lcd_setuserfont_r(volatile u08 *font);
extern void lcd_setuserfont_r(u08 *font);
extern void lcd_setuserfont_f(const char *font);

/*
** macros for automatically storing string constant in program memory
*/
#ifndef P
#define P(s) ({static const char c[] __attribute__ ((progmem)) = s;c;})
#endif
#define lcd_puts_P(__s)         lcd_puts_f(P(__s))


#endif //LCD_H

/*-----------------------------------------------------------------------*/
/* lcd.h */